@@ -112,9 +112,9 @@ public enum RealtimeExceptionCodes implements OXExceptionCode {
     STANZA_REMOTE_SERVER_TIMEOUT(RealtimeExceptionMessages.STANZA_REMOTE_SERVER_TIMEOUT_MSG, CATEGORY_SERVICE_DOWN, 18),
     STANZA_RESOURCE_CONSTRAINT(RealtimeExceptionMessages.STANZA_RESOURCE_CONSTRAINT_MSG, CATEGORY_SERVICE_DOWN, 19),
     STANZA_SERVICE_UNAVAILABLE(RealtimeExceptionMessages.NEEDED_SERVICE_MISSING_MSG, CATEGORY_SERVICE_DOWN, 20),
-    STANZA_SUBSCRIPTION_REQUIRED(RealtimeExceptionMessages.NEEDED_SERVICE_MISSING_MSG, CATEGORY_SERVICE_DOWN, 21),
-    STANZA_UNDEFINED_CONDITION(RealtimeExceptionMessages.NEEDED_SERVICE_MISSING_MSG, CATEGORY_SERVICE_DOWN, 22),
-    STANZA_UNEXPECTED_REQUEST(RealtimeExceptionMessages.NEEDED_SERVICE_MISSING_MSG, CATEGORY_SERVICE_DOWN, 23),
+    STANZA_SUBSCRIPTION_REQUIRED(RealtimeExceptionMessages.STANZA_SUBSCRIPTION_REQUIRED_MSG, CATEGORY_SERVICE_DOWN, 21),
+    STANZA_UNDEFINED_CONDITION(RealtimeExceptionMessages.STANZA_UNDEFINED_CONDITION_MSG, CATEGORY_SERVICE_DOWN, 22),
+    STANZA_UNEXPECTED_REQUEST(RealtimeExceptionMessages.STANZA_UNEXPECTED_REQUEST_MSG, CATEGORY_SERVICE_DOWN, 23),
 
     // Atmosphere
     
@@ -133,8 +133,10 @@ public enum RealtimeExceptionCodes implements OXExceptionCode {
     SESSION_INVALID(RealtimeExceptionMessages.SESSION_INVALID_MSG, CATEGORY_ERROR, 1005, STANZA_NOT_ACCEPTABLE, STANZA_NOT_ACCEPTABLE),
     /** Invalid sequence detected.*/
     SEQUENCE_INVALID(RealtimeExceptionMessages.SEQUENCE_INVALID_MSG, CATEGORY_ERROR, 1006, STANZA_BAD_REQUEST, STANZA_BAD_REQUEST),
+    /** The server has no state associated with this client. Please enrol first. */
+    STATE_MISSING(RealtimeExceptionMessages.STATE_MISSING_MSG, CATEGORY_ERROR, 1007, STANZA_BAD_REQUEST, STANZA_BAD_REQUEST)
     ;
-    
+
     private int number;
 
     private Category category;
